var LiveEdit;
(function (LiveEdit) {
    var DomHelper = (function () {
        function DomHelper() {
        }
        DomHelper.getDocumentSize = function () {
            return {
                width: wemjq(document).width(),
                height: wemjq(document).height()
            };
        };
        DomHelper.getViewPortSize = function () {
            var win = wemjq(window);
            return {
                width: win.width(),
                height: win.height()
            };
        };
        DomHelper.getDocumentScrollTop = function () {
            return wemjq(document).scrollTop();
        };
        DomHelper.supportsTouch = function () {
            return document.hasOwnProperty('ontouchend');
        };
        return DomHelper;
    })();
    LiveEdit.DomHelper = DomHelper;
})(LiveEdit || (LiveEdit = {}));
var LiveEdit;
(function (LiveEdit) {
    var PageViewBuilder = api.liveedit.PageViewBuilder;
    var ComponentViewDragStartedEvent = api.liveedit.ComponentViewDragStartedEvent;
    var ComponentViewDragStoppedEvent = api.liveedit.ComponentViewDragStoppedEvent;
    var ComponentResetEvent = api.liveedit.ComponentResetEvent;
    var ItemViewIdProducer = api.liveedit.ItemViewIdProducer;
    var Shader = api.liveedit.Shader;
    var Highlighter = api.liveedit.Highlighter;
    var SelectedHighlighter = api.liveedit.SelectedHighlighter;
    var Cursor = api.liveedit.Cursor;
    var DragAndDrop = api.liveedit.DragAndDrop;
    var Exception = api.Exception;
    var LiveEditPage = (function () {
        function LiveEditPage() {
            var _this = this;
            this.skipNextReloadConfirmation = false;
            api.liveedit.SkipLiveEditReloadConfirmationEvent.on(function (event) {
                _this.skipNextReloadConfirmation = event.isSkip();
            });
            api.liveedit.InitializeLiveEditEvent.on(function (event) {
                var liveEditModel = event.getLiveEditModel();
                var body = api.dom.Body.get().loadExistingChildren();
                try {
                    _this.pageView = new PageViewBuilder().
                        setItemViewProducer(new ItemViewIdProducer()).
                        setLiveEditModel(liveEditModel).
                        setElement(body).
                        build();
                }
                catch (error) {
                    if (api.ObjectHelper.iFrameSafeInstanceOf(error, Exception)) {
                        new api.liveedit.LiveEditPageInitializationErrorEvent('The Live edit page could not be initialized. ' +
                            error.getMessage()).fire();
                    }
                    else {
                        new api.liveedit.LiveEditPageInitializationErrorEvent('The Live edit page could not be initialized. ' +
                            error).fire();
                    }
                    return;
                }
                DragAndDrop.init(_this.pageView);
                api.ui.Tooltip.allowMultipleInstances(false);
                _this.registerGlobalListeners();
                new api.liveedit.LiveEditPageViewReadyEvent(_this.pageView).fire();
            });
        }
        LiveEditPage.prototype.registerGlobalListeners = function () {
            var _this = this;
            api.dom.WindowDOM.get().onBeforeUnload(function (event) {
                if (!_this.skipNextReloadConfirmation) {
                    var message = "This will close this wizard!";
                    (event || window.event)['returnValue'] = message;
                    return message;
                }
            });
            api.dom.WindowDOM.get().onUnload(function (event) {
                if (!_this.skipNextReloadConfirmation) {
                    new api.liveedit.PageUnloadedEvent(_this.pageView).fire();
                }
                else {
                    _this.skipNextReloadConfirmation = false;
                }
                _this.pageView.remove();
            });
            api.liveedit.ComponentLoadedEvent.on(function (event) {
                if (api.liveedit.layout.LayoutItemType.get().equals(event.getNewComponentView().getType())) {
                    DragAndDrop.get().createSortableLayout(event.getNewComponentView());
                }
                else {
                    DragAndDrop.get().refreshSortable();
                }
            });
            ComponentResetEvent.on(function (event) {
                DragAndDrop.get().refreshSortable();
            });
            ComponentViewDragStartedEvent.on(function () {
                Highlighter.get().hide();
                SelectedHighlighter.get().hide();
                Shader.get().hide();
                Cursor.get().hide();
            });
            ComponentViewDragStoppedEvent.on(function () {
                Cursor.get().reset();
                if (_this.pageView.isLocked()) {
                    Highlighter.get().hide();
                    Shader.get().shade(_this.pageView);
                }
            });
        };
        return LiveEditPage;
    })();
    LiveEdit.LiveEditPage = LiveEditPage;
})(LiveEdit || (LiveEdit = {}));
api.StyleHelper.setCurrentPrefix(api.StyleHelper.PAGE_EDITOR_PREFIX);
var liveEditPage;
wemjq(document).ready(function () {
    if (liveEditPage) {
        return;
    }
    liveEditPage = new LiveEdit.LiveEditPage();
    wemjq(document).on("keypress keydown keyup", function (event) {
        if ((event.metaKey || event.ctrlKey || event.altKey) && event.keyCode) {
            stopBrowserShortcuts(event);
            wemjq(parent.document).simulate(event.type, {
                bubbles: event.bubbles,
                cancelable: event.cancelable,
                view: parent,
                ctrlKey: event.ctrlKey,
                altKey: event.altKey,
                shiftKey: event.shiftKey,
                metaKey: event.metaKey,
                keyCode: event.keyCode,
                charCode: event.charCode
            });
        }
    });
    function stopBrowserShortcuts(event) {
        var activeBindings = parent['api']['ui']['KeyBindings'].get().getActiveBindings();
        var hasMatch = hasMatchingBinding(activeBindings, event);
        if (hasMatch) {
            event.preventDefault();
            console.log('Prevented default for event in live edit because it has binding in parent', event);
        }
    }
    function hasMatchingBinding(keys, event) {
        var isMod = event.ctrlKey || event.metaKey;
        var isAlt = event.altKey;
        var key = event.keyCode || event.which;
        for (var i = 0; i < keys.length; i++) {
            var matches = false;
            switch (keys[i].getCombination()) {
                case 'backspace':
                    matches = key == 8;
                    break;
                case 'del':
                    matches = key == 46;
                case 'mod+del':
                    matches = matches && isMod;
                    break;
                case 'mod+s':
                    matches = key == 83 && isMod;
                    break;
                case 'mod+esc':
                    matches = key == 83 && isMod;
                    break;
                case 'mod+alt+f4':
                    matches = key == 115 && isMod && isAlt;
                    break;
            }
            if (matches) {
                return true;
            }
        }
        return false;
    }
});
// Globals
///<reference path='../../common/js/_all.d.ts' />
///<reference path='shared/DomHelper.ts' />
///<reference path='LiveEditPage.ts' />
///<reference path='main.ts' />
//# sourceMappingURL=_all.js.map